import json
import requests
import gradio as gr
import time
from ast import literal_eval

system_msg = "You are an expert Biomedical Research Scientist."
basic_instruction = """Your job is to extract the following biomedical entities from a given sentence:
a. Spans/entities that refer to DNA
b. Spans/entities that refer to Proteins
c. Spans/entities that refer to cell types
d. Spans/entities that refer to cell lines
e. Spans/entities that refer to RNA
If you cannot find all of the aforementioned spans/entities in the sentanece you can leave non found entities empty."""

def format_input_text_prompt(in_sent: str) -> str:
  text= (f"<|begin_of_text|><|start_header_id|>system<|end_header_id|>\n\n"
  f"{system_msg}<|eot_id|><|start_header_id|>user<|end_header_id|>\n\n"
  f"Below is an instruction that describes a task. Write a response that appropriately completes the request.\n\n### Instruction:\n{basic_instruction}\n\n### Input:\n{in_sent}\n\n### Response:\n"
  f"<|eot_id|><|start_header_id|>asssitant<|end_header_id|>\n\n")

  return "".join(text)

def call_tgi_client(text_input: str, end_point_url: str = "https://q31t62mdx1jfy2-80.proxy.runpod.net") -> str:

    start = time.time()
    prompt_input = format_input_text_prompt(text_input)
    print(prompt_input)
    payload = json.dumps({
        "inputs": prompt_input,
        "parameters": {
            "max_new_tokens": 256, "temperature": 0.01, "do_sample": False, "return_full_text": False,
            'stop': ["<|end_of_text|>", "<|eot_id|>"]
        }
    })
    headers = {'Content-Type': 'application/json'}

    response = requests.request("POST", end_point_url, headers=headers, data=payload)

    print("Done in {:.2f}s".format(time.time()-start))
    return literal_eval(response.text)[0]["generated_text"]


gr_theme = gr.themes.Default(
    font=['Noto Sans', 'Helvetica', 'ui-sans-serif', 'system-ui', 'sans-serif'],
    font_mono=['IBM Plex Mono', 'ui-monospace', 'Consolas', 'monospace'],
).set(
    border_color_primary='#c5c5d2',
    button_large_padding='6px 12px',
    body_text_color_subdued='#484848',
    background_fill_secondary='#eaeaea'
)

demo = gr.Interface(fn=call_tgi_client,inputs=gr.Textbox(label="Input text:"),outputs=gr.Textbox(label="Answer (inference):"),title='Finetuned LLama3-8b-Instruct model for biomedical NER')
demo.launch(share=True)
